@@ -21,7 +21,6 @@ test_that("save_piece_obj works", {
 
 test_that("rgl works", {
     skip_on_cran()
-    skip_on_os("mac")
     library("rgl")
     # go stone OBJ generation uses "rgl" in background
     files <- save_piece_obj("bit_face", cfg = game_systems("dejavu3d")$go)
@@ -30,19 +29,16 @@ test_that("rgl works", {
     files <- save_piece_obj("pawn_top", cfg = game_systems("sans3d", pawn="peg-doll")$piecepack)
     expect_length(files, 3)
 
-    rgl.open()
+    open3d(useNULL = TRUE)
     df <- tibble::tibble(piece_side = "tile_face", scale = c(1, 0))
     pmap_piece(df, piece3d, cfg=cfg)
-    f <- tempfile(fileext = ".png")
-    rgl.snapshot(f)
-    expect_true(file.exists(f))
-    rgl.close()
-    unlink(f)
 
+    clear3d()
     cfg <- game_systems("sans3d", pawn = "joystick")$piecepack
-    rgl.clear()
     l <- piece3d("pawn_top", x=-1:1, suit=1:3, cfg = cfg, lit=TRUE)
     expect_true(length(l) > 2)
+
+    close3d()
 })
 
 test_that("rayrender works", {
